from .SimpleImageReader import readImageSimple

advanced = True
try:
    from .AdvancedImageReader import readImageAdvanced
except ModuleNotFoundError:
    advanced = False


def readImage(image_file: str, output_file: str, depth: int) -> bytes:
    if advanced:
        return readImageAdvanced(image_file, output_file, depth)
    else:
        return readImageSimple(image_file, output_file, depth)



import tkinter
from .RawReader import readRaw

img = None
tk = tkinter.Tk()
tk.withdraw()


def readImageSimple(image_file: str, output_file: str, depth: int) -> bytes:
    global img, tk
    img = tkinter.PhotoImage(file=image_file, master=tk)
    tk.image = img
    pixels = []
    for index in range(img.width() * img.height()):
        pixels.append(img.get(index % img.width(), index // img.width()))
    pixels = [channel for pixel in pixels for channel in pixel]

    byte_string = readRaw(pixels, depth)

    try:
        with open(output_file, "wb") as f:
            f.write(byte_string)
    except TypeError:
        pass

    return byte_string



import array
import math
from typing import Union


def readRaw(initial_data: Union[array.array, list], depth: int) -> bytes:
    bit_file_data = []
    byteslist = []

    bit_depth = int(math.log2(depth))
    dec_remainder = 2 ** bit_depth

    for index in range(0, len(initial_data)):
        data = format(initial_data[index] % dec_remainder, "b")
        data = "0" * (bit_depth - len(data)) + data
        for bit in data:
            bit_file_data.append(int(bit))
    for index in range(0, len(bit_file_data), 8):
        byte = ""
        for bit in bit_file_data[index:index+8]:
            byte += str(bit)
        byteslist.append(int(byte, 2))
    return bytes(byteslist)



import wave
import array
from .RawReader import readRaw


def readAudio(audio_file: str, output_file: str, depth: int) -> bytes:
    with wave.open(audio_file, 'rb') as out_wavefile:
        sizes = {1: 'B', 2: 'h', 4: 'i'}
        samp_size = out_wavefile.getsampwidth()
        frmt = sizes[samp_size]

        initial_data = array.array(frmt)
        initial_data.frombytes(out_wavefile.readframes(out_wavefile.getnframes()))

    byte_string = readRaw(initial_data, depth)

    try:
        with open(output_file, "wb") as f:
            f.write(byte_string)
    except TypeError:
        pass

    return byte_string


if __name__ == "__main__":
    location2 = "C:\\Users\\Alexander\\PycharmProjects\\StegaSaurus\\src\\"
    readAudio(location2 + 'sine2.wav', location2 + "Untitled2.jpg", 2)



from .RawWriter import writeRaw
import array
try:
    from PIL import Image
except ModuleNotFoundError:
    raise ModuleNotFoundError("Please install PIL with pip install PIL or use SimpleImageReader for non-jpeg images")


def writeImageAdvanced(in_image_file: str, out_image_file: str, data_file: str, depth: int) -> list:
    img = Image.open(in_image_file)
    img = img.convert("RGB")
    pixels = list(img.getdata())
    pixels = [channel for pixel in pixels for channel in pixel]

    initial_data = array.array('B')
    initial_file_data = array.array('B')
    file_data_bits = array.array('B')
    initial_data.fromlist(pixels)

    with open(data_file, "rb") as f:
        file_data = f.read()
    initial_file_data.frombytes(file_data)
    for byte in initial_file_data:
        for bit in range(7, -1, -1):
            file_data_bits.append(byte >> bit & 1)
    for _ in range(len(initial_data) - len(file_data_bits)):
        file_data_bits.append(0)

    final_data = writeRaw(initial_data, file_data_bits, depth)

    position = 0
    newimg = Image.new('RGB', (img.width, img.height), "black")
    for row in range(img.height):
        for col in range(img.width):
            newimg.putpixel((col, row), (final_data[position], final_data[position + 1], final_data[position + 2]))
            position += 3
    newimg.save(out_image_file)

    return final_data.tolist()



import math
import array
from typing import Union


def writeRaw(initial_data: array.array, file_data_bits: Union[array.array, list], depth: int) -> array.array:
    final_data = array.array(initial_data.typecode)
    bit_depth = int(math.log2(depth))
    dec_remainder = 2 ** bit_depth

    for index in range(0, len(initial_data)):
        addition = ""
        for bit in file_data_bits[index * bit_depth:(index + 1) * bit_depth]:
            addition += str(bit)
        if addition == "":
            addition = "0"
        final_data.append(initial_data[index] // dec_remainder * dec_remainder + int(addition, 2))
    return final_data



from .RawReader import readRaw
try:
    from PIL import Image
except ModuleNotFoundError:
    raise ModuleNotFoundError("Please install PIL with pip install PIL or use SimpleImageReader for non-jpeg images")


def readImageAdvanced(image_file: str, output_file: str, depth: int) -> bytes:
    im = Image.open(image_file)
    im = im.convert("RGB")
    pixels = list(im.getdata())
    pixels = [channel for pixel in pixels for channel in pixel]

    byte_string = readRaw(pixels, depth)

    try:
        with open(output_file, "wb") as f:
            f.write(byte_string)
    except TypeError:
        pass

    return byte_string



import wave
import array
from .RawWriter import writeRaw


def writeAudio(in_audio_file: str, out_audio_file: str, data_file: str, depth: int) -> list:
    with wave.open(in_audio_file, 'rb') as wavefile:
        sizes = {1: 'B', 2: 'h', 4: 'i'}
        channels = wavefile.getnchannels()
        samp_size = wavefile.getsampwidth()
        frmt = sizes[samp_size]

        initial_data = array.array(frmt)
        initial_file_data = array.array('B')
        file_data_bits = array.array('B')
        initial_data.frombytes(wavefile.readframes(wavefile.getnframes()))

        with open(data_file, "rb") as f:
            file_data = f.read()
        initial_file_data.frombytes(file_data)
        for byte in initial_file_data:
            for bit in range(7, -1, -1):
                file_data_bits.append(byte >> bit & 1)
        for _ in range(len(initial_data) - len(file_data_bits)):
            file_data_bits.append(0)

        final_data = writeRaw(initial_data, file_data_bits, depth)

        with wave.open(out_audio_file, 'wb') as out_wavefile:
            out_wavefile.setnchannels(channels)
            out_wavefile.setsampwidth(samp_size)
            out_wavefile.setframerate(wavefile.getframerate())
            out_wavefile.setnframes(len(final_data))
            out_wavefile.setcomptype(wavefile.getcomptype(), wavefile.getcompname())
            out_wavefile.writeframesraw(final_data.tobytes())

    return final_data.tolist()


if __name__ == "__main__":
    location2 = "C:\\Users\\Alexander\\PycharmProjects\\StegaSaurus\\src\\"
    writeAudio(location2 + 'sine.wav', location2 + 'sine2.wav', location2 + "Untitled.jpg", 2)



from .RawWriter import writeRaw
import array
import tkinter

img = None
tk = tkinter.Tk()
tk.withdraw()


def writeImageSimple(in_image_file: str, out_image_file: str, data_file: str, depth: int) -> list:
    global img, tk
    img = tkinter.PhotoImage(file=in_image_file, master=tk)
    tk.image = img
    pixels = []
    for index in range(img.width() * img.height()):
        pixels.append(img.get(index % img.width(), index // img.width()))
    pixels = [channel for pixel in pixels for channel in pixel]

    initial_data = array.array('B')
    initial_file_data = array.array('B')
    file_data_bits = array.array('B')
    initial_data.fromlist(pixels)

    with open(data_file, "rb") as f:
        file_data = f.read()
    initial_file_data.frombytes(file_data)
    for byte in initial_file_data:
        for bit in range(7, -1, -1):
            file_data_bits.append(byte >> bit & 1)
    for _ in range(len(initial_data) - len(file_data_bits)):
        file_data_bits.append(0)

    final_data = writeRaw(initial_data, file_data_bits, depth)

    position = 0
    for row in range(img.height()):
        for col in range(img.width()):
            colour = "{#%02x%02x%02x}" % (final_data[position], final_data[position + 1], final_data[position + 2])
            img.put(colour, (col, row))
            position += 3
    img.write(out_image_file, format=out_image_file[-3:])

    return final_data.tolist()



from .SimpleImageWriter import writeImageSimple

advanced = True
try:
    from .AdvancedImageWriter import writeImageAdvanced
except ModuleNotFoundError:
    advanced = False


def writeImage(in_image_file: str, out_image_file: str, data_file: str, depth: int) -> list:
    if advanced:
        return writeImageAdvanced(in_image_file, out_image_file, data_file, depth)
    else:
        return writeImageSimple(in_image_file, out_image_file, data_file, depth)
